@@ -529,7 +529,7 @@ newgame()
 	(void) signal(SIGINT, (SIG_RET_TYPE) done1);
 #endif
 #ifdef NEWS
-	if(iflags.news) display_file(NEWS, FALSE);
+	if(iflags.news) display_file_area(NEWS_AREA, NEWS, FALSE);
 #endif
 	load_qtlist();	/* load up the quest text info */
 /*	quest_init();*/	/* Now part of role_init() */
