@@ -270,6 +270,23 @@ doaltarobj(obj)  /* obj is an object dropped on an altar */
 			otense(obj, "land"));
 		obj->bknown = 1;
 	}
+	/* Also BUC one level deep inside containers */
+	if (Has_contents(obj)) {
+		int bcucount = 0;
+		struct obj *otmp;
+		for (otmp = obj->cobj; otmp; otmp = otmp->nobj) {
+			if (otmp->blessed || otmp->cursed)
+				bcucount++;
+			if (!Hallucination) otmp->bknown = 1;
+		}
+		if (bcucount == 1) {
+			pline("Looking inside %s, you see a colored flash.",
+					the(xname(obj)));
+		} else if (bcucount > 1) {
+			pline("Looking inside %s, you see colored flashes.",
+					the(xname(obj)));
+		}
+	}
 }
 
 #ifdef SINKS
@@ -774,7 +791,7 @@ int retry;
 	all_categories = FALSE;
 	n = query_category("Drop what type of items?",
 			invent,
-			UNPAID_TYPES | ALL_TYPES | CHOOSE_ALL |
+			UNIDENTIFIED_TYPES | UNPAID_TYPES | ALL_TYPES | CHOOSE_ALL |
 			BUC_BLESSED | BUC_CURSED | BUC_UNCURSED | BUC_UNKNOWN,
 			&pick_list, PICK_ANY);
 	if (!n) goto drop_done;
@@ -1093,6 +1110,7 @@ boolean at_stairs, falling, portal;
 	boolean new = FALSE;	/* made a new level? */
 	struct monst *mtmp;
 	char whynot[BUFSZ];
+	char *annotation;
 
 	if (dunlev(newlevel) > dunlevs_in_dungeon(newlevel))
 		newlevel->dlevel = dunlevs_in_dungeon(newlevel);
@@ -1340,6 +1358,12 @@ boolean at_stairs, falling, portal;
 
 	initrack();
 
+#ifdef RECORD_ACHIEVE
+#ifdef LIVELOGFILE
+	livelog_achieve_update();
+#endif
+#endif
+
 	if ((mtmp = m_at(u.ux, u.uy)) != 0
 #ifdef STEED
 		&& mtmp != u.usteed
@@ -1484,6 +1508,12 @@ boolean at_stairs, falling, portal;
 	    onquest();
 	assign_level(&u.uz0, &u.uz); /* reset u.uz0 */
 
+	/* show level annotation when entering the level */
+	if (iflags.show_annotation &&
+	    (annotation = get_annotation(&u.uz))) {
+		You("annotated this level: %s", annotation);
+	}
+
 #ifdef INSURANCE
 	save_currentstate();
 #endif
