@@ -9,6 +9,7 @@
 
 #ifdef OVL1
 
+#define DUNGEON_AREA    FILE_AREA_UNSHARE
 #define DUNGEON_FILE	"dungeon"
 
 #define X_START		"x-strt"
@@ -64,9 +65,9 @@ mapseen *mapseenchn = (struct mapseen *)0;
 STATIC_DCL mapseen *FDECL(load_mapseen, (int));
 STATIC_DCL void FDECL(save_mapseen, (int, mapseen *));
 STATIC_DCL mapseen *FDECL(find_mapseen, (d_level *));
-STATIC_DCL void FDECL(print_mapseen, (winid,mapseen *,boolean));
+STATIC_DCL void FDECL(print_mapseen, (winid,mapseen *, boolean, boolean, boolean));
 STATIC_DCL boolean FDECL(interest_mapseen, (mapseen *));
-STATIC_DCL char *FDECL(seen_string, (xchar x, const char *));
+STATIC_DCL char *FDECL(seen_string, (xchar, const char *));
 STATIC_DCL const char *FDECL(br_string2, (branch *));
 
 #ifdef DEBUG
@@ -744,7 +745,7 @@ init_dungeons()		/* initialize the "dungeon" structs */
 
 	pd.n_levs = pd.n_brs = 0;
 
-	dgn_file = dlb_fopen(DUNGEON_FILE, RDBMODE);
+	dgn_file = dlb_fopen_area(DUNGEON_AREA, DUNGEON_FILE, RDBMODE);
 	if (!dgn_file) {
 	    char tbuf[BUFSZ];
 	    Sprintf(tbuf, "Cannot open dungeon description - \"%s",
@@ -1921,7 +1922,11 @@ donamelevel()
 
 	if (!(mptr = find_mapseen(&u.uz))) return 0;
 
-	Sprintf(qbuf,"What do you want to call this dungeon level? ");
+	if (mptr->custom) {
+		Sprintf(qbuf,"Replace previous annotation \"%s\" with?", mptr->custom);
+	} else {
+		Sprintf(qbuf,"What do you want to call this dungeon level?");
+	}
 	getlin(qbuf, nbuf);
 
 	if (index(nbuf, '\033')) return 0;
@@ -2129,7 +2134,7 @@ recalc_mapseen()
 {
 	mapseen *mptr;
 	struct monst *shkp;
-	int x, y, ridx;
+	unsigned x, y, ridx;
 
 	/* Should not happen in general, but possible if in the process
 	 * of being booted from the quest.  The mapseen object gets
@@ -2156,6 +2161,10 @@ recalc_mapseen()
 	/* recalculate room knowledge: for now, just shops and temples
 	 * this could be extended to an array of 0..SHOPBASE
 	 */
+#ifdef BLACKMARKET
+	/* kludge for not recording the shop room on the blackmarket level */
+	if (!Is_blackmarket(&u.uz))
+#endif
 	for (x = 0; x < sizeof(mptr->rooms); x++) {
 		if (mptr->rooms[x] & MSR_SEEN) {
 			if (rooms[x].rtype >= SHOPBASE) {
@@ -2225,8 +2234,8 @@ recalc_mapseen()
 				break;
 			case ALTAR:
 				if (!mptr->feat.naltar)
-					mptr->feat.msalign = Amask2msa(levl[x][y].altarmask);
-				else if (mptr->feat.msalign != Amask2msa(levl[x][y].altarmask))
+					mptr->feat.msalign = Amask2msa(levl[x][y].altarmask & ~AM_SHRINE);
+				else if (mptr->feat.msalign != Amask2msa(levl[x][y].altarmask & ~AM_SHRINE))
 					mptr->feat.msalign = MSA_NONE;
 						
 				mptr->feat.naltar = min(mptr->feat.naltar + 1, 3);
@@ -2258,7 +2267,7 @@ dooverview()
 		if (interest_mapseen(mptr)) {
 			printdun = (first || lastdun != mptr->lev.dnum);
 			/* if (!first) putstr(win, 0, ""); */
-			print_mapseen(win, mptr, printdun);
+			print_mapseen(win, mptr, printdun, FALSE, FALSE);
 
 			if (printdun) {
 				first = FALSE;
@@ -2273,6 +2282,56 @@ dooverview()
 	return 0;
 }
 
+#ifdef DUMP_LOG
+int
+dumpoverview()
+{
+	mapseen *mptr;
+	boolean printdun;
+	int lastdun=-1;
+	boolean first = TRUE;
+	boolean previous_was_interesting = FALSE;
+
+	first = TRUE;
+
+	/* lazy intialization */
+	(void) recalc_mapseen();
+
+	for (mptr = mapseenchn; mptr; mptr = mptr->next) {
+		/* try to find out if the last branch printed something */
+		if (!first && lastdun != mptr->lev.dnum && previous_was_interesting) {
+			dump_html("</li a>\n", "");
+			previous_was_interesting = FALSE;
+		}
+		/* only print out info for a level or a dungeon if interest */
+		if (interest_mapseen(mptr)) {
+			previous_was_interesting = TRUE;
+			printdun = (first || lastdun != mptr->lev.dnum);
+
+			if (first) {
+				/* Always print header as there will at least
+				 * be the output of the current level */
+				dump_title("Dungeon overview");
+				dump_list_start();
+			}
+			/* currently dumping is always at the end of the game */
+			print_mapseen(0, mptr, printdun, TRUE, TRUE);
+
+			if (printdun) {
+				first = FALSE;
+				lastdun = mptr->lev.dnum;
+			}
+		}
+	}
+	if (!first) {
+		dump_html("</li>\n", "");
+		dump_list_end();
+	}
+
+	return 0;
+}
+#endif
+
 STATIC_OVL char *
 seen_string(x, obj)
 xchar x;
@@ -2366,10 +2425,12 @@ int rtype;
 #define ADDTOBUF(nam, var) { if (var) Sprintf(eos(buf), "%s " nam, COMMA); }
 
 STATIC_OVL void
-print_mapseen(win, mptr, printdun)
+print_mapseen(win, mptr, printdun, dump, final)
 winid win;
 mapseen *mptr;
 boolean printdun;
+boolean dump; /**< if information should be dumped to file */
+boolean final; /**< if game is finished */
 {
 	char buf[BUFSZ];
 	int i, depthstart;
@@ -2381,7 +2442,7 @@ boolean printdun;
 	if (mptr->lev.dnum == quest_dnum || mptr->lev.dnum == knox_level.dnum)
 		depthstart = 1;
 	else
-		depthstart = dungeons[mptr->lev.dnum].depth_start;  
+		depthstart = dungeons[mptr->lev.dnum].depth_start;
 
 	if (printdun) {
 		/* Sokoban lies about dunlev_ureached and we should
@@ -2395,9 +2456,19 @@ boolean printdun;
 				dungeons[mptr->lev.dnum].dname,
 				depthstart, depthstart + 
 				dungeons[mptr->lev.dnum].dunlev_ureached - 1);
-		putstr(win, ATR_INVERSE, buf);
+		if (!dump) {
+			putstr(win, ATR_INVERSE, buf);
+		} else {
+#ifdef DUMP_LOG
+			dump_text("  %s\n", buf);
+			dump_html("<li>%s\n", buf);
+#endif
+		}
 	}
 
+#ifdef DUMP_LOG
+	dump_definition_list_start();
+#endif
 	/* calculate level number */
 	i = depthstart + mptr->lev.dlevel - 1;
 	if (Is_astralevel(&mptr->lev))
@@ -2416,7 +2487,7 @@ boolean printdun;
 	
 #ifdef WIZARD
 	/* wizmode prints out proto dungeon names for clarity */
-	if (wizard) {
+	if (wizard || final) {
 		s_level *slev;
 		if ((slev = Is_special(&mptr->lev)))
 			Sprintf(eos(buf), " [%s]", slev->proto);
@@ -2428,10 +2499,21 @@ boolean printdun;
 
 	/* print out glyph or something more interesting? */
 	Sprintf(eos(buf), "%s", on_level(&u.uz, &mptr->lev) ? 
-		" <- You are here" : "");
-	putstr(win, ATR_BOLD, buf);
+		(final ? " <- You were here" : " <- You are here") : "");
+	if (!dump) {
+		putstr(win, ATR_BOLD, buf);
+	} else {
+#ifdef DUMP_LOG
+		dump_definition_list_dt(buf);
+#endif
+	}
 
-	if (mptr->feat.forgot) return;
+	if (mptr->feat.forgot) {
+#ifdef DUMP_LOG
+		if (dump) dump_definition_list_end();
+#endif
+		return;
+	}
 
 	if (INTEREST(mptr->feat)) {
 		buf[0] = 0;
@@ -2471,7 +2553,13 @@ boolean printdun;
 		i = strlen(PREFIX);
 		*buf = highc(*buf);
 
-		putstr(win, 0, buf);
+		if (!dump) {
+			putstr(win, 0, buf);
+		} else {
+#ifdef DUMP_LOG
+			dump_definition_list_dd(buf);
+#endif
+		}
 	}
 
 	/* print out branches */
@@ -2485,7 +2573,31 @@ boolean printdun;
 		 */
 		if (mptr->br->end1_up && !In_endgame(&(mptr->br->end2)))
 			Sprintf(eos(buf), ", level %d", depth(&(mptr->br->end2)));
-		putstr(win, 0, buf);
+		if (!dump) {
+			putstr(win, 0, buf);
+		} else {
+#ifdef DUMP_LOG
+			dump_definition_list_dd(buf);
+#endif
+		}
+	}
+
+#ifdef DUMP_LOG
+	if (dump) dump_definition_list_end();
+#endif
+}
+
+/** Get annotation for a specific level. */
+char *
+get_annotation(lev)
+d_level *lev;
+{
+	mapseen *mptr;
+
+	if (!(mptr = find_mapseen(&u.uz))) {
+		return NULL;
+	} else {
+		return mptr->custom;
 	}
 }
 
