@@ -24,13 +24,17 @@ static NEARDATA const char readable[] =
 static const char all_count[] = { ALLOW_COUNT, ALL_CLASSES, 0 };
 
 static void FDECL(wand_explode, (struct obj *));
+#if 0
 static void NDECL(do_class_genocide);
+#endif
 static void FDECL(stripspe,(struct obj *));
 static void FDECL(p_glow1,(struct obj *));
 static void FDECL(p_glow2,(struct obj *,const char *));
 static void FDECL(randomize,(int *, int));
 static void FDECL(forget_single_object, (int));
+#if 0
 static void FDECL(forget, (int));
+#endif
 static void FDECL(maybe_tame, (struct monst *,struct obj *));
 
 STATIC_PTR void FDECL(do_flood, (int,int,genericptr_t));
@@ -685,6 +689,7 @@ forget_map(howmuch)
 	    if (howmuch & ALL_MAP || rn2(7)) {
 		/* Zonk all memory of this location. */
 		levl[zx][zy].seenv = 0;
+		levl[zx][zy].stepped_on = 0;
 		levl[zx][zy].waslit = 0;
 		levl[zx][zy].glyph = cmap_to_glyph(S_stone);
 		levl[zx][zy].styp = STONE;
@@ -752,6 +757,7 @@ forget_levels(percent)
 	}
 }
 
+#if 0
 /*
  * Forget some things (e.g. after reading a scroll of amnesia).  When called,
  * the following are always forgotten:
@@ -794,6 +800,7 @@ int howmuch;
 	 */
 	docrt();		/* this correctly will reset vision */
 }
+#endif
 
 /* monster is hit by scroll of taming's effect */
 static void
@@ -1510,7 +1517,7 @@ register struct obj	*sobj;
 		    return 0;
 		}
 		(void) create_gas_cloud(cc.x, cc.y, 3+bcsign(sobj),
-						8+4*bcsign(sobj));
+						8+4*bcsign(sobj), rn1(3,4));
 		break;
 	}
 	default:
@@ -1642,6 +1649,7 @@ do_it:
 	vision_full_recalc = 1;	/* delayed vision recalculation */
 }
 
+#if 0
 static void
 do_class_genocide()
 {
@@ -1794,6 +1802,7 @@ do_class_genocide()
 		return;
 	}
 }
+#endif
 
 #define REALLY 1
 #define PLAYER 2
@@ -1906,14 +1915,15 @@ boolean only_on_level; /**< if TRUE only genocide monsters on current level,
 		which = !type_is_pname(ptr) ? "the " : "";
 	}
 	if (how & REALLY) {
+	    char* genocided_name = (*which != 'a') ? buf : makeplural(buf);
 	    /* setting no-corpse affects wishing and random tin generation */
 	    if (!only_on_level) { mvitals[mndx].mvflags |= (G_GENOD | G_NOCORPSE); }
 	    pline("Wiped out %s%s%s.", which,
-		  (*which != 'a') ? buf : makeplural(buf),
+		  genocided_name,
 		  on_this_level);
 
 #ifdef LIVELOGFILE
-	livelog_genocide(buf, only_on_level);
+	livelog_genocide(genocided_name, only_on_level);
 #endif
 
 	if (killplayer) {
