@@ -157,14 +157,14 @@ dosave0()
 #endif
 
 	HUP if (iflags.window_inited) {
-	    uncompress(fq_save);
+	    uncompress_area(fq_save, SAVEF);
 	    fd = open_savefile();
 	    if (fd > 0) {
 		(void) close(fd);
 		clear_nhwindow(WIN_MESSAGE);
 		There("seems to be an old save file.");
 		if (yn("Overwrite the old file?") == 'n') {
-		    compress(fq_save);
+		    compress_area(fq_save, SAVEF);
 		    return 0;
 		}
 	    }
@@ -289,7 +289,7 @@ dosave0()
 	/* get rid of current level --jgm */
 	delete_levelfile(ledger_no(&u.uz));
 	delete_levelfile(0);
-	compress(fq_save);
+	compress_area(FILE_AREA_SAVE, fq_save);
 	return(1);
 }
 
@@ -526,7 +526,8 @@ int mode;
 			&& prm->lit == rgrm->lit
 			&& prm->waslit == rgrm->waslit
 			&& prm->roomno == rgrm->roomno
-			&& prm->edge == rgrm->edge) {
+			&& prm->edge == rgrm->edge
+			&& prm->stepped_on == rgrm->stepped_on) {
 			match++;
 			if (match > 254) {
 			    match = 254;	/* undo this match */
@@ -918,7 +919,6 @@ save_lvl_sounds(fd, or, mode)
 register int fd, mode;
 register struct lvl_sounds *or;
 {
-    struct lvl_sound_bite *mt;
     int marker = 0;
     int i;
     int len;
@@ -1128,6 +1128,7 @@ freedynamicdata()
 	unload_qtlist();
 	free_invbuf();	/* let_to_name (invent.c) */
 	free_youbuf();	/* You_buf,&c (pline.c) */
+	msgpline_free();
 #ifdef MENU_COLOR
 	free_menu_coloring();
 #endif
