@@ -79,6 +79,7 @@ STATIC_DCL int FDECL(score_wanted,
 #ifdef RECORD_ACHIEVE
 STATIC_DCL long FDECL(encodeachieve, (void));
 #endif
+STATIC_DCL long FDECL(encode_xlogflags, (void));
 #ifdef NO_SCAN_BRACK
 STATIC_DCL void FDECL(nsb_mung_line,(char*));
 STATIC_DCL void FDECL(nsb_unmung_line,(char*));
@@ -283,6 +284,7 @@ struct toptenentry *tt;
                 "version=%s-%d.%d.%d"
                 SEP "points=%ld"
                 SEP "deathdnum=%d"
+                SEP "deathdname=%s"
                 SEP "deathlev=%d"
                 SEP "maxlvl=%d"
                 SEP "dlev_name=%s"
@@ -294,7 +296,7 @@ struct toptenentry *tt;
                 SEP "uid=%d",
                 GAME_SHORT_NAME,
                 tt->ver_major, tt->ver_minor, tt->patchlevel,
-                tt->points, tt->deathdnum, tt->deathlev,
+                tt->points, tt->deathdnum, dungeons[tt->deathdnum].dname, tt->deathlev,
                 tt->maxlvl,
                 lev ? lev->proto : "", /* proto level name if special level */
                 tt->hp, tt->maxhp, tt->deaths,
@@ -313,6 +315,8 @@ struct toptenentry *tt;
    munge_xlstring(buf, tt->death, DTHSZ + 1);
   (void)fprintf(rfile, SEP "death=%s", buf);
 
+  (void)fprintf(rfile, SEP "flags=0x%lx", encode_xlogflags());
+
 #ifdef RECORD_CONDUCT
   (void)fprintf(rfile, SEP "conduct=0x%lx", encodeconduct());
 #endif
@@ -468,26 +472,34 @@ int how;
 #endif
 
 #ifdef LOGFILE		/* used for debugging (who dies of what, where) */
+#ifdef FILE_AREAS
+	if (lock_file_area(LOGAREA, LOGFILE, 10)) {
+#else
 	if (lock_file(LOGFILE, SCOREPREFIX, 10)) {
-	    if(!(lfile = fopen_datafile(LOGFILE, "a", SCOREPREFIX))) {
+#endif
+	    if(!(lfile = fopen_datafile_area(LOGAREA, LOGFILE, "a", SCOREPREFIX))) {
 		HUP raw_print("Cannot open log file!");
 	    } else {
 		writeentry(lfile, t0);
 		(void) fclose(lfile);
 	    }
-	    unlock_file(LOGFILE);
+	    unlock_file_area(LOGAREA, LOGFILE);
 	}
 #endif /* LOGFILE */
 
 #ifdef XLOGFILE
-	if(lock_file(XLOGFILE, SCOREPREFIX, 10)) {
-		if(!(xlfile = fopen_datafile(XLOGFILE, "a", SCOREPREFIX))) {
+#ifdef FILE_AREAS
+	if (lock_file_area(LOGAREA, XLOGFILE, 10)) {
+#else
+	if (lock_file(XLOGFILE, SCOREPREFIX, 10)) {
+#endif
+		if(!(xlfile = fopen_datafile_area(LOGAREA, XLOGFILE, "a", SCOREPREFIX))) {
 			HUP raw_print("Cannot open extended log file!");
 		} else {
 			write_xlentry(xlfile, t0);
 			(void) fclose(xlfile);
 		}
-		unlock_file(XLOGFILE);
+		unlock_file_area(LOGAREA, XLOGFILE);
 	}
 #endif /* XLOGFILE */
 
@@ -509,18 +521,22 @@ int how;
 	    goto showwin;
 	}
 
+#ifdef FILE_AREAS
+	if (!lock_file_area(NH_RECORD_AREA, RECORD, 60))
+#else
 	if (!lock_file(RECORD, SCOREPREFIX, 60))
+#endif
 		goto destroywin;
 
 #ifdef UPDATE_RECORD_IN_PLACE
-	rfile = fopen_datafile(RECORD, "r+", SCOREPREFIX);
+	rfile = fopen_datafile_area(NH_RECORD_AREA, RECORD, "r+", SCOREPREFIX);
 #else
-	rfile = fopen_datafile(RECORD, "r", SCOREPREFIX);
+	rfile = fopen_datafile_area(NH_RECORD_AREA, RECORD, "r", SCOREPREFIX);
 #endif
 
 	if (!rfile) {
 		HUP raw_print("Cannot open record file!");
-		unlock_file(RECORD);
+		unlock_file_area(NH_RECORD_AREA, RECORD);
 		goto destroywin;
 	}
 
@@ -599,9 +615,9 @@ int how;
 				     t0->fpos : final_fpos), SEEK_SET);
 #else
 		(void) fclose(rfile);
-		if(!(rfile = fopen_datafile(RECORD, "w", SCOREPREFIX))){
+		if(!(rfile = fopen_datafile_area(NH_RECORD_AREA, RECORD, "w", SCOREPREFIX))){
 			HUP raw_print("Cannot write record file");
-			unlock_file(RECORD);
+			unlock_file_area(NH_RECORD_AREA, RECORD);
 			free_ttlist(tt_head);
 			goto destroywin;
 		}
@@ -697,7 +713,7 @@ int how;
 	}
 #endif	/* UPDATE_RECORD_IN_PLACE */
 	(void) fclose(rfile);
-	unlock_file(RECORD);
+	unlock_file_area(NH_RECORD_AREA, RECORD);
 	free_ttlist(tt_head);
 
   showwin:
@@ -948,6 +964,20 @@ int uid;
 	return 0;
 }
 
+long
+encode_xlogflags(void)
+{
+	long e = 0L;
+
+	if (wizard)              e |= 0x001L; /* wizard mode */
+	if (discover)            e |= 0x002L; /* explore mode */
+	if (killer_flags & 0x1)  e |= 0x004L; /* died, (with the Amulet) */
+	if (killer_flags & 0x2)  e |= 0x008L; /* died, (in celestial disgrace) */
+	if (killer_flags & 0x4)  e |= 0x010L; /* died, (with a fake Amulet) */
+
+	return e;
+}
+
 #ifdef RECORD_CONDUCT
 long
 encodeconduct(void)
@@ -1040,7 +1070,7 @@ char **argv;
 		return;
 	}
 
-	rfile = fopen_datafile(RECORD, "r", SCOREPREFIX);
+	rfile = fopen_datafile_area(NH_RECORD_AREA, RECORD, "r", SCOREPREFIX);
 	if (!rfile) {
 		raw_print("Cannot open record file!");
 		return;
@@ -1198,7 +1228,7 @@ struct obj *otmp;
 
 	if (!otmp) return((struct obj *) 0);
 
-	rfile = fopen_datafile(RECORD, "r", SCOREPREFIX);
+	rfile = fopen_datafile_area(NH_RECORD_AREA, RECORD, "r", SCOREPREFIX);
 	if (!rfile) {
 		impossible("Cannot open record file!");
 		return (struct obj *)0;
