@@ -110,7 +110,7 @@ void curses_init_nhwindows(int* argcp, char** argv)
     noecho();
     raw();
     meta(stdscr, TRUE);
-    curs_set(0);
+    orig_cursor = curs_set(0);
     keypad(stdscr, TRUE);
 #ifdef NCURSES_VERSION
 # ifdef __APPLE__
@@ -207,6 +207,8 @@ void curses_get_nh_event()
 */
 void curses_exit_nhwindows(const char *str)
 {
+    curses_cleanup();
+    curs_set(orig_cursor);
     endwin();
     iflags.window_inited = 0;
     if (str != NULL)
@@ -218,6 +220,7 @@ void curses_exit_nhwindows(const char *str)
 /* Prepare the window to be suspended. */
 void curses_suspend_nhwindows(const char *str)
 {
+    endwin();
 }
 
 
@@ -336,10 +339,17 @@ void curses_putstr(winid wid, int attr, const char *text)
 /* Display the file named str.  Complain about missing files
                    iff complain is TRUE.
 */
+#ifdef FILE_AREAS
+void curses_display_file(const char *filearea,const char *filename,BOOLEAN_P must_exist)
+{
+    curses_view_file(filearea, filename, must_exist);
+}
+#else
 void curses_display_file(const char *filename,BOOLEAN_P must_exist)
 {
     curses_view_file(filename, must_exist);
 }
+#endif
 
 /* Start using window as a menu.  You must call start_menu()
    before add_menu().  After calling start_menu() you may not
