@@ -129,6 +129,9 @@ static char obuf[BUFSIZ];	/* BUFSIZ is defined in stdio.h */
 static char winpanicstr[] = "Bad window id %d";
 char defmorestr[] = "--More--";
 
+/** Track if the player is still selecting his character. */
+boolean in_character_selection = FALSE;
+
 #ifdef MENU_COLOR
 extern struct menucoloring *menu_colorings;
 #endif
@@ -333,6 +336,8 @@ tty_player_selection()
 	anything any;
 	menu_item *selected = 0;
 
+	in_character_selection = TRUE;
+
 	/* prevent an unnecessary prompt */
 	rigid_role_checks();
 
@@ -709,6 +714,7 @@ give_up:	/* Quit */
 	    }
 	}
 	/* Success! */
+	in_character_selection = FALSE;
 	tty_display_nhwindow(BASE_WINDOW, FALSE);
 }
 
@@ -1189,14 +1195,42 @@ invert_all(window, page_start, page_end, acc)
     }
 }
 
+/* Strip [] from strings like "a [cursed] rusty iron wand" so
+ * menucolors regexp can apply.
+ * Completely hacky. */
+static void
+strip_brackets(str)
+char *str;
+{
+	/* from http://stackoverflow.com/questions/4161822 */
+	char *src, *dest;
+
+	src = dest = str; /* both pointers point to the first char of input */
+	while (*src != '\0') /* exit loop when null terminator reached */
+	{
+		if (*src != '[' && *src != ']') /* if source is not a [] char */
+		{
+			*dest = *src; /* copy the char at source to destination */
+			dest++;       /* increment destination pointer */
+		}
+		src++; /* increment source pointer */
+	}
+	*dest = '\0'; /* terminate string with null terminator */
+}
+
 #ifdef MENU_COLOR
 boolean
-get_menu_coloring(str, color, attr)
-const char *str;
+get_menu_coloring(line, color, attr)
+const char *line;
 int *color, *attr;
 {
     struct menucoloring *tmpmc;
-    if (iflags.use_menu_color)
+    char str[BUFSZ];
+
+    strcpy(str, line);
+    strip_brackets(str);
+
+    if (iflags.use_menu_color && iflags.use_color)
 	for (tmpmc = menu_colorings; tmpmc; tmpmc = tmpmc->next)
 # ifdef MENU_COLOR_REGEX
 #  ifdef MENU_COLOR_REGEX_POSIX
@@ -1311,7 +1345,7 @@ struct WinDesc *cw;
 		     * this.
 		     */
 #ifdef MENU_COLOR
-		   if (iflags.use_menu_color &&
+		   if (iflags.use_menu_color && iflags.use_color &&
 		       (menucolr = get_menu_coloring(curr->str, &color,&attr))) {
 		       term_start_attr(attr);
 		       if (color != NO_COLOR) term_start_color(color);
@@ -1335,7 +1369,7 @@ struct WinDesc *cw;
 			} else
 			    (void) putchar(*cp);
 #ifdef MENU_COLOR
-		   if (iflags.use_menu_color && menucolr) {
+		   if (iflags.use_menu_color && iflags.use_color && menucolr) {
 		       if (color != NO_COLOR) term_end_color();
 		       term_end_attr(attr);
 		   } else
@@ -1825,7 +1859,15 @@ tty_putsym(window, x, y, ch)
     case NHW_MAP:
     case NHW_BASE:
 	tty_curs(window, x, y);
+#ifdef UTF8_GLYPHS
+	if (iflags.UTF8graphics) {
+		pututf8char(ch);
+	} else {
+		(void) putchar(ch);
+	}
+#else
 	(void) putchar(ch);
+#endif
 	ttyDisplay->curx++;
 	cw->curx++;
 	break;
@@ -2009,14 +2051,23 @@ tty_putstr(window, attr, str)
 }
 
 void
+#ifdef FILE_AREAS
+tty_display_file(farea, fname, complain)
+const char *farea;
+#else
 tty_display_file(fname, complain)
+#endif
 const char *fname;
 boolean complain;
 {
 #ifdef DEF_PAGER			/* this implies that UNIX is defined */
     {
 	/* use external pager; this may give security problems */
+#ifdef FILE_AREAS
+	register int fd = open_area(farea, fname, 0, 0);
+#else
 	register int fd = open(fname, 0);
+#endif
 
 	if(fd < 0) {
 	    if(complain) pline("Cannot open %s.", fname);
@@ -2046,7 +2097,11 @@ boolean complain;
 	char *cr;
 
 	tty_clear_nhwindow(WIN_MESSAGE);
+#ifdef FILE_AREAS
+	f = dlb_fopen_area(farea, fname, "r");
+#else
 	f = dlb_fopen(fname, "r");
+#endif
 	if (!f) {
 	    if(complain) {
 		home();  tty_mark_synch();  tty_raw_print("");
@@ -2412,7 +2467,8 @@ docorner(xmin, ymax)
     }
 
     end_glyphout();
-    if (ymax >= (int) wins[WIN_STATUS]->offy) {
+    if (!in_character_selection && /* check for status lines to update */
+        (ymax >= (int) wins[WIN_STATUS]->offy)) {
 					/* we have wrecked the bottom line */
 	flags.botlx = 1;
 	bot();
@@ -2578,7 +2634,15 @@ tty_print_glyph(window, x, y, glyph)
       xputg(glyph,ch,special);
     else
 #endif
+#ifdef UTF8_GLYPHS
+	if (iflags.UTF8graphics) {
+		pututf8char(ch);
+	} else {
+		g_putch(ch);	/* print the character */
+	}
+#else
 	g_putch(ch);		/* print the character */
+#endif
 
     if (reverse_on) {
     	term_end_attr(ATR_INVERSE);
