@@ -120,7 +120,7 @@ char *str;
 int *color, *attr;
 {
     struct menucoloring *tmpmc;
-    if (iflags.use_menu_color)
+    if (iflags.use_menu_color && iflags.use_color)
 	for (tmpmc = menu_colorings; tmpmc; tmpmc = tmpmc->next)
 # ifdef MENU_COLOR_REGEX
 #  ifdef MENU_COLOR_REGEX_POSIX
@@ -844,6 +844,16 @@ BOOL onDrawItem(HWND hWnd, WPARAM wParam, LPARAM lParam)
 	OldFg = SetTextColor(lpdis->hDC, 
 		menu_fg_brush ? menu_fg_color : (COLORREF)GetSysColor(DEFAULT_COLOR_FG_MENU)); 
 
+#ifdef MENU_COLOR
+	if (iflags.use_menu_color &&
+		iflags.use_color &&
+		(menucolr = get_menu_coloring(item->str, &color,&attr))) {
+		/* TODO: use attr too */
+		if (color != NO_COLOR)
+		SetTextColor(lpdis->hDC, nhcolor_to_RGB(color));
+	}
+#endif
+
     GetTextMetrics(lpdis->hDC, &tm);
 
 	x = lpdis->rcItem.left + 1;
@@ -873,16 +883,6 @@ BOOL onDrawItem(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		if(item->accelerator!=0) {
 			buf[0] = item->accelerator;
 			buf[1] = '\x0';
-
-#ifdef MENU_COLOR
-			if (iflags.use_menu_color &&
-			    (menucolr = get_menu_coloring(item->str, &color,&attr))) {
-			    /* TODO: use attr too */
-			    if (color != NO_COLOR)
-				SetTextColor(lpdis->hDC, nhcolor_to_RGB(color));
-			}
-#endif
-
 			SetRect( &drawRect, x, lpdis->rcItem.top, lpdis->rcItem.right, lpdis->rcItem.bottom );
 			DrawText(lpdis->hDC, NH_A2W(buf, wbuf, 2), 1, &drawRect, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
 		}
